use serde::{Serialize, Deserialize};
use serde_json::{Value, json};
#[macro_use]
mod modules;
pub use crate::modules::{cpu, devices, external, gpu, memory, meta, network, os, storage, uptime};
use thiserror::Error;

#[derive(Debug, Serialize, Deserialize)]
pub struct TelemetryInfo {
    telemetry: AllInfo,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct AllInfo {
    gpu: gpu::GPUInfo,
    cpu: Vec<cpu::CPUInfo>,
    memory: memory::MemoryInfo,
    os: os::OSInfo,
    storage: Vec<storage::StorageInfo>,
    network: network::NetworkInfo,
    external: external::ExternalInfo,
    devices: Vec<devices::DeviceInfo>,
    meta: meta::MetaInfo,
    uptime: uptime::UptimeInfo,
}

#[derive(Debug, Error)]
pub enum TelemetryError {
    // #[error("Failed to get GPU information")]
    // GPUError(#[from] gpu::GPUError),
    #[error("Failed to get CPU information")]
    CPUError(#[from] cpu::CPUError),
    #[error("Failed to get OS information")]
    OSError(#[from] os::OSError),
    #[error("Failed to get storage information")]
    StorageError(#[from] storage::StorageError),
    // #[error("Failed to get external information")]
    // ExternalError(#[from] external::ExternalError),
    #[error("Failed to get devices information")]
    DevicesError(#[from] devices::DevicesError),
    #[error("Failed to get meta information")]
    MetaError(#[from] meta::MetaError),
    #[error("Failed to get uptime information")]
    UptimeError(#[from] uptime::UptimeError),
    #[error("Failed to build JSON for TelemetryInfo")]
    JsonError(#[from] serde_json::Error),
}

#[allow(dead_code)]
pub fn get_struct() -> Result<TelemetryInfo, TelemetryError> {
    Ok(TelemetryInfo {
        telemetry: AllInfo {
            gpu: gpu::get_struct(),
            cpu: cpu::get_struct()?,
            memory: memory::get_struct(),
            os: os::get_struct()?,
            storage: storage::get_list()?,
            network: network::get_struct(),
            external: external::get_struct(),
            devices: devices::get_struct()?,
            meta: meta::get_struct()?,
            uptime: uptime::get_struct()?,
        },
    })
}

#[allow(dead_code)]
pub fn get_json() -> Result<Value, TelemetryError> {
    Ok(json!(get_struct()?))
}
