use std::str::FromStr;

use serde::{Serialize, Deserialize};
use detect_desktop_environment::DesktopEnvironment;
use strum_macros::EnumString;

#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash, Serialize, Deserialize, EnumString)]
#[strum(serialize_all = "PascalCase", ascii_case_insensitive)]
#[non_exhaustive]
pub enum FmtDE {
    Cinnamon,
    Cosmic,
    CosmicEpoch,
    Dde,
    Ede,
    Endless,
    Enlightenment,
    Gnome,
    Hyprland,
    Kde,
    Lxde,
    Lxqt,
    MacOs,
    Mate,
    Old,
    Pantheon,
    Razor,
    Rox,
    Sway,
    Tde,
    Unity,
    Windows,
    Xfce,
}

impl FmtDE {
    pub fn detect() -> Option<Self> {
        DesktopEnvironment::detect().and_then(|inner_de| {
            let s = format!("{:?}", inner_de);
            
            FmtDE::from_str(&s).ok()
        })
    }
}