use crate::modules::FmtVersion;
use serde::{Serialize, Deserialize};
use serde_json::{Value, json};
use sys_locale::get_locale;
use thiserror::Error;
use versions::Versioning;

include!(concat!(env!("OUT_DIR"), "/built.rs"));

#[derive(Debug, Serialize, Deserialize)]
pub struct MetaInfo {
    version: FmtVersion,
    commit_hash: String,
    locale: String,
}

#[derive(Debug, Error)]
pub enum MetaError {
    #[error("Invalid Semver version")]
    InvalidVersion,
    #[error("Failed to build JSON for MetaInfo")]
    JsonError(#[from] serde_json::Error),
}

#[allow(dead_code)]
pub fn get_struct() -> Result<MetaInfo, MetaError> {
    let locale = get_locale().unwrap_or_else(|| String::from("en-US"));
    let version = Versioning::new(env!("CARGO_PKG_VERSION")).ok_or(MetaError::InvalidVersion)?;

    Ok(MetaInfo {
        version: FmtVersion(version),
        commit_hash: GIT_COMMIT_HASH.to_string(),
        locale,
    })
}

#[allow(dead_code)]
pub fn get_json() -> Result<Value, MetaError> {
    Ok(json!(get_struct()?))
}
